#include "qae_u.h"
#include <errno.h>

typedef struct ms_qae_appraise_quote_result_t {
	quote3_error_t ms_retval;
	uint8_t* ms_wasm_buf;
	size_t ms_wasm_size;
	const char* ms_p_verification_result_token;
	size_t ms_p_verification_result_token_len;
	uint8_t** ms_p_qaps;
	uint8_t ms_qaps_count;
	time_t ms_appraisal_check_date;
	sgx_ql_qe_report_info_t* ms_p_qae_report_info;
	uint32_t* ms_p_appraisal_result_token_buffer_size;
	uint8_t** ms_p_appraisal_result_token;
} ms_qae_appraise_quote_result_t;

typedef struct ms_qae_authenticate_appraisal_result_t {
	quote3_error_t ms_retval;
	const uint8_t* ms_p_quote;
	uint32_t ms_quote_size;
	const char* ms_p_appraisal_result_token;
	size_t ms_p_appraisal_result_token_len;
	const tee_policy_bundle_t* ms_p_policies;
	tee_policy_auth_result_t* ms_result;
	sgx_ql_qe_report_info_t* ms_p_qae_report_info;
} ms_qae_authenticate_appraisal_result_t;

typedef struct ms_qae_authenticate_policy_owner_t {
	quote3_error_t ms_retval;
	const uint8_t* ms_p_quote;
	uint32_t ms_quote_size;
	const char* ms_p_appraisal_result_token;
	size_t ms_p_appraisal_result_token_len;
	const uint8_t** ms_policy_key_list;
	uint32_t ms_list_size;
	tee_policy_auth_result_t* ms_result;
	sgx_ql_qe_report_info_t* ms_p_qae_report_info;
} ms_qae_authenticate_policy_owner_t;

typedef struct ms_ocall_malloc_t {
	int ms_retval;
	uint8_t** ms_buf;
	uint32_t ms_buf_size;
} ms_ocall_malloc_t;

typedef struct ms_ocall_get_default_platform_policy_t {
	quote3_error_t ms_retval;
	const uint8_t* ms_fmspc;
	uint32_t ms_fmspc_size;
	uint8_t** ms_pp_default_platform_policy;
	uint32_t* ms_default_platform_policy_size;
} ms_ocall_get_default_platform_policy_t;

typedef struct ms_ocall_free_default_platform_policy_t {
	quote3_error_t ms_retval;
	uint8_t* ms_p_default_platform_policy;
	uint32_t ms_default_policy_size;
} ms_ocall_free_default_platform_policy_t;

typedef struct ms_sgx_oc_cpuidex_t {
	int* ms_cpuinfo;
	int ms_leaf;
	int ms_subleaf;
} ms_sgx_oc_cpuidex_t;

typedef struct ms_sgx_thread_wait_untrusted_event_ocall_t {
	int ms_retval;
	const void* ms_self;
} ms_sgx_thread_wait_untrusted_event_ocall_t;

typedef struct ms_sgx_thread_set_untrusted_event_ocall_t {
	int ms_retval;
	const void* ms_waiter;
} ms_sgx_thread_set_untrusted_event_ocall_t;

typedef struct ms_sgx_thread_setwait_untrusted_events_ocall_t {
	int ms_retval;
	const void* ms_waiter;
	const void* ms_self;
} ms_sgx_thread_setwait_untrusted_events_ocall_t;

typedef struct ms_sgx_thread_set_multiple_untrusted_events_ocall_t {
	int ms_retval;
	const void** ms_waiters;
	size_t ms_total;
} ms_sgx_thread_set_multiple_untrusted_events_ocall_t;

typedef struct ms_pthread_wait_timeout_ocall_t {
	int ms_retval;
	unsigned long long ms_waiter;
	unsigned long long ms_timeout;
} ms_pthread_wait_timeout_ocall_t;

typedef struct ms_pthread_create_ocall_t {
	int ms_retval;
	unsigned long long ms_self;
} ms_pthread_create_ocall_t;

typedef struct ms_pthread_wakeup_ocall_t {
	int ms_retval;
	unsigned long long ms_waiter;
} ms_pthread_wakeup_ocall_t;

static sgx_status_t SGX_CDECL qae_ocall_malloc(void* pms)
{
	ms_ocall_malloc_t* ms = SGX_CAST(ms_ocall_malloc_t*, pms);
	ms->ms_retval = ocall_malloc(ms->ms_buf, ms->ms_buf_size);

	return SGX_SUCCESS;
}

static sgx_status_t SGX_CDECL qae_ocall_get_default_platform_policy(void* pms)
{
	ms_ocall_get_default_platform_policy_t* ms = SGX_CAST(ms_ocall_get_default_platform_policy_t*, pms);
	ms->ms_retval = ocall_get_default_platform_policy(ms->ms_fmspc, ms->ms_fmspc_size, ms->ms_pp_default_platform_policy, ms->ms_default_platform_policy_size);

	return SGX_SUCCESS;
}

static sgx_status_t SGX_CDECL qae_ocall_free_default_platform_policy(void* pms)
{
	ms_ocall_free_default_platform_policy_t* ms = SGX_CAST(ms_ocall_free_default_platform_policy_t*, pms);
	ms->ms_retval = ocall_free_default_platform_policy(ms->ms_p_default_platform_policy, ms->ms_default_policy_size);

	return SGX_SUCCESS;
}

static sgx_status_t SGX_CDECL qae_sgx_oc_cpuidex(void* pms)
{
	ms_sgx_oc_cpuidex_t* ms = SGX_CAST(ms_sgx_oc_cpuidex_t*, pms);
	sgx_oc_cpuidex(ms->ms_cpuinfo, ms->ms_leaf, ms->ms_subleaf);

	return SGX_SUCCESS;
}

static sgx_status_t SGX_CDECL qae_sgx_thread_wait_untrusted_event_ocall(void* pms)
{
	ms_sgx_thread_wait_untrusted_event_ocall_t* ms = SGX_CAST(ms_sgx_thread_wait_untrusted_event_ocall_t*, pms);
	ms->ms_retval = sgx_thread_wait_untrusted_event_ocall(ms->ms_self);

	return SGX_SUCCESS;
}

static sgx_status_t SGX_CDECL qae_sgx_thread_set_untrusted_event_ocall(void* pms)
{
	ms_sgx_thread_set_untrusted_event_ocall_t* ms = SGX_CAST(ms_sgx_thread_set_untrusted_event_ocall_t*, pms);
	ms->ms_retval = sgx_thread_set_untrusted_event_ocall(ms->ms_waiter);

	return SGX_SUCCESS;
}

static sgx_status_t SGX_CDECL qae_sgx_thread_setwait_untrusted_events_ocall(void* pms)
{
	ms_sgx_thread_setwait_untrusted_events_ocall_t* ms = SGX_CAST(ms_sgx_thread_setwait_untrusted_events_ocall_t*, pms);
	ms->ms_retval = sgx_thread_setwait_untrusted_events_ocall(ms->ms_waiter, ms->ms_self);

	return SGX_SUCCESS;
}

static sgx_status_t SGX_CDECL qae_sgx_thread_set_multiple_untrusted_events_ocall(void* pms)
{
	ms_sgx_thread_set_multiple_untrusted_events_ocall_t* ms = SGX_CAST(ms_sgx_thread_set_multiple_untrusted_events_ocall_t*, pms);
	ms->ms_retval = sgx_thread_set_multiple_untrusted_events_ocall(ms->ms_waiters, ms->ms_total);

	return SGX_SUCCESS;
}

static sgx_status_t SGX_CDECL qae_pthread_wait_timeout_ocall(void* pms)
{
	ms_pthread_wait_timeout_ocall_t* ms = SGX_CAST(ms_pthread_wait_timeout_ocall_t*, pms);
	ms->ms_retval = pthread_wait_timeout_ocall(ms->ms_waiter, ms->ms_timeout);

	return SGX_SUCCESS;
}

static sgx_status_t SGX_CDECL qae_pthread_create_ocall(void* pms)
{
	ms_pthread_create_ocall_t* ms = SGX_CAST(ms_pthread_create_ocall_t*, pms);
	ms->ms_retval = pthread_create_ocall(ms->ms_self);

	return SGX_SUCCESS;
}

static sgx_status_t SGX_CDECL qae_pthread_wakeup_ocall(void* pms)
{
	ms_pthread_wakeup_ocall_t* ms = SGX_CAST(ms_pthread_wakeup_ocall_t*, pms);
	ms->ms_retval = pthread_wakeup_ocall(ms->ms_waiter);

	return SGX_SUCCESS;
}

static const struct {
	size_t nr_ocall;
	void * table[11];
} ocall_table_qae = {
	11,
	{
		(void*)qae_ocall_malloc,
		(void*)qae_ocall_get_default_platform_policy,
		(void*)qae_ocall_free_default_platform_policy,
		(void*)qae_sgx_oc_cpuidex,
		(void*)qae_sgx_thread_wait_untrusted_event_ocall,
		(void*)qae_sgx_thread_set_untrusted_event_ocall,
		(void*)qae_sgx_thread_setwait_untrusted_events_ocall,
		(void*)qae_sgx_thread_set_multiple_untrusted_events_ocall,
		(void*)qae_pthread_wait_timeout_ocall,
		(void*)qae_pthread_create_ocall,
		(void*)qae_pthread_wakeup_ocall,
	}
};
sgx_status_t qae_appraise_quote_result(sgx_enclave_id_t eid, quote3_error_t* retval, uint8_t* wasm_buf, size_t wasm_size, const char* p_verification_result_token, uint8_t** p_qaps, uint8_t qaps_count, time_t appraisal_check_date, sgx_ql_qe_report_info_t* p_qae_report_info, uint32_t* p_appraisal_result_token_buffer_size, uint8_t** p_appraisal_result_token)
{
	sgx_status_t status;
	ms_qae_appraise_quote_result_t ms;
	ms.ms_wasm_buf = wasm_buf;
	ms.ms_wasm_size = wasm_size;
	ms.ms_p_verification_result_token = p_verification_result_token;
	ms.ms_p_verification_result_token_len = p_verification_result_token ? strlen(p_verification_result_token) + 1 : 0;
	ms.ms_p_qaps = p_qaps;
	ms.ms_qaps_count = qaps_count;
	ms.ms_appraisal_check_date = appraisal_check_date;
	ms.ms_p_qae_report_info = p_qae_report_info;
	ms.ms_p_appraisal_result_token_buffer_size = p_appraisal_result_token_buffer_size;
	ms.ms_p_appraisal_result_token = p_appraisal_result_token;
	status = sgx_ecall(eid, 0, &ocall_table_qae, &ms);
	if (status == SGX_SUCCESS && retval) *retval = ms.ms_retval;
	return status;
}

sgx_status_t qae_authenticate_appraisal_result(sgx_enclave_id_t eid, quote3_error_t* retval, const uint8_t* p_quote, uint32_t quote_size, const char* p_appraisal_result_token, const tee_policy_bundle_t* p_policies, tee_policy_auth_result_t* result, sgx_ql_qe_report_info_t* p_qae_report_info)
{
	sgx_status_t status;
	ms_qae_authenticate_appraisal_result_t ms;
	ms.ms_p_quote = p_quote;
	ms.ms_quote_size = quote_size;
	ms.ms_p_appraisal_result_token = p_appraisal_result_token;
	ms.ms_p_appraisal_result_token_len = p_appraisal_result_token ? strlen(p_appraisal_result_token) + 1 : 0;
	ms.ms_p_policies = p_policies;
	ms.ms_result = result;
	ms.ms_p_qae_report_info = p_qae_report_info;
	status = sgx_ecall(eid, 1, &ocall_table_qae, &ms);
	if (status == SGX_SUCCESS && retval) *retval = ms.ms_retval;
	return status;
}

sgx_status_t qae_authenticate_policy_owner(sgx_enclave_id_t eid, quote3_error_t* retval, const uint8_t* p_quote, uint32_t quote_size, const char* p_appraisal_result_token, const uint8_t** policy_key_list, uint32_t list_size, tee_policy_auth_result_t* result, sgx_ql_qe_report_info_t* p_qae_report_info)
{
	sgx_status_t status;
	ms_qae_authenticate_policy_owner_t ms;
	ms.ms_p_quote = p_quote;
	ms.ms_quote_size = quote_size;
	ms.ms_p_appraisal_result_token = p_appraisal_result_token;
	ms.ms_p_appraisal_result_token_len = p_appraisal_result_token ? strlen(p_appraisal_result_token) + 1 : 0;
	ms.ms_policy_key_list = policy_key_list;
	ms.ms_list_size = list_size;
	ms.ms_result = result;
	ms.ms_p_qae_report_info = p_qae_report_info;
	status = sgx_ecall(eid, 2, &ocall_table_qae, &ms);
	if (status == SGX_SUCCESS && retval) *retval = ms.ms_retval;
	return status;
}

